% Script:     spectrum_demo_2D
% Copyright (c) 2019 by S. Lawrence Marple Jr.

% This script demonstrates two-dimensional spectral analysis by one of five
% techniques:  classical periodogram, classical Blackman-Tukey, quarter-plane
% autoregressive by the two-dimensional extensions of the Yule-Walker method
% and lattice method of least squares linear prediction, and the minimum
% variance PSD method.

disp(' >> CONSULT THE README FILE BEFORE FIRST USE OF THIS SCRIPT <<');
disp(' Press space bar to continue. ');  % 6 June 2019

disp(' ');
disp(' This script demonstrates 2-D spectral analysis techniques.');
disp(' ');

%*********************************  Data Sources  ******************************

disp(' ***** TWO-DIMENSIONAL DATA SOURCE *****')
disp(' 2-D data choices for two-dimensional spectral analysis demonstration:')
disp('     1 -- 8 x 8 2-D signal: three 2-D complex sines + 2-D white noise');
disp('     2 -- 423 x 427 texture image');
disp('     3 -- 8 x 8 2-D signal: two 2-D real sines + additive noise');
dataselect = input(' Enter number of 2-D data source choice: ');

if dataselect == 1
   x = zeros(8,8)+1i*zeros(8,8);
   for j=1:8
      for k=1:8
        x(j,k)=exp(1i*2*pi*(.3*j+.3*k)) + exp(1i*2*pi*(-.2*j-.2*k)) +...
            exp(1i*2*pi*(.2*j+.4*k));
      end
   end
   x = x + .001*(randn([8,8]) + 1i*randn([8,8]));
   % x-axis and y-axis spatial sampling intervals in cm/sample; spatial
   % frequency domain interpreted therefore in cycles/cm units
   Tx = 0.1;
   Ty = 0.1;
elseif dataselect == 2
   x = double(rgb2gray(imread('highfreqtexture.jpg','jpg'))); % convert to gray scale
   [mx,my] = size(x);
   x = x - sum(sum(x))/(mx*my);    % remove DC offset in image
   % x-axis and y-axis spatial sampling intervals in pixels per sample;
   % spatial frequency domain interpreted therefore in cycles/pixel units
   Tx = 1;
   Ty = 1;
elseif dataselect == 3
   x = zeros(8,8);
   for j=1:8
      for k=1:8
        x(j,k)=sin(2*pi*(.3*j+.3*k)) + sin(2*pi*(-.2*j-.2*k));
      end
   end
   x = x+.001*randn([8,8]);
   % x-axis and y-axis spatial sampling intervals in inches per sample;
   % example in which each axis has a different sampling interval;
   % spatial frequency domain interpreted therefore in cycles/inch units
   Tx = 0.2;
   Ty = 0.3;
else
    error(' Choice number is incorrect.')
end
num_psdx = 512; % number of frequency points of 2-D PSD along rows (x axis)
num_psdy = 512; % number of frequency points of 2-D PSD along columns (y axis)
disp(' ');

%*******************  Two - Dimensional Spectral Analysis  *********************

disp(' ***** 2-D SPECTRAL ANALYSIS TECHNIQUE *****')
disp(' Two-dimensional spectral analysis technique choices:')
disp('      1 -- two-dimensional Welch periodogram method')
disp('      2 -- two-dimensional Blackman-Tukey correlogram method')
disp('      3 -- two-dimensional autoregressive Yule-Walker method')
disp('      4 -- two-dimensional autoregressive lattice method')
disp('      5 -- two-dimensional minimum variance PSD method')
method = input(' Enter number of algorithm choice: ');
disp(' ')

disp(' ***** 2-D SPECTRAL ANALYSIS PARAMETERS *****')
%***  Two-Dimensional Periodogram Spectral Analysis  ***
if method == 1
   seg_sizex = input(' Enter row subarray dimension per periodogram (#samples): ');
   if  seg_sizex > num_psdx
			    error([' Segment size cannot exceed NUM_PSDx = ',int2str(num_psdx)])
   end
   seg_sizey = input(' Enter column subarray dimension per periodogram (#samples): ');
   if  seg_sizey > num_psdy
			    error([' Segment size cannot exceed NUM_PSDy = ',int2str(num_psdy)])
   end
   overlapx = input(' Enter inter-subarray overlap along rows (#samples): ');
   if  (overlapx < 0) || (overlapx >= seg_sizex)
       error([' Segment overlap outside range  0<=OVERLAPx<',int2str(seg_sizex)])
   end
   overlapy = input(' Enter inter-subarray overlap along columns (#samples): ');
   if  (overlapy < 0) || (overlapy >= seg_sizey)
       error([' Segment overlap outside range  0<=OVERLAPy<',int2str(seg_sizey)])
   end
   window = input(' Enter window choice-- 0 [None], 1 [Hamming], 2 [Nuttall]: ');
   disp(' ');
   disp(' ***** COMPUTING 2-D SPECTRUM BASED ON SELECTIONS *****');  
   psd2D = periodogram_psd_2D(num_psdx,num_psdy,window,...
                                 overlapx,overlapy,seg_sizex,seg_sizey,Tx,Ty,x);

%***  Two-Dimensional (Correlogram) Blackman - Tukey Spectral Analysis  ***
elseif method == 2
   max_lagx=input(' Enter maximum row lag (# samples) for estimated 2-D ACS: ');
   max_lagy=input(' Enter maximum column lag (# samp) for estimated 2-D ACS: ');
   disp(' ');
   disp(' ***** COMPUTING 2-D SPECTRUM BASED ON SELECTIONS *****');  
   psd2D = correlogram_psd_2D(num_psdx,num_psdy,max_lagx,max_lagy,Tx,Ty,x);

%***  Two-Dimensional Autoregressive Spectral Analysis (Yule-Walker)  ***
elseif method == 3
   px = input(' Enter row order of two-dimensional autoregressive process: ');
   py = input(' Enter column order of two-dimensional autoregressive process: ');
   disp(' ');
   disp(' ***** COMPUTING 2-D SPECTRUM BASED ON SELECTIONS *****');  
   [rho_Q1,a_Q1,rho_Q4,a_Q4] = yule_walker_2D(px,py,x);
   psd_type = 3; %input(' Enter psd type (1--Q1, 2--Q4, 3--combined Q1 & Q4): ');
   psd2D = ar_psd_2D(psd_type,num_psdx,num_psdy,Tx,Ty,rho_Q1,a_Q1,rho_Q4,a_Q4);

%***  Two-Dimensional Autoregressive Spectral Analysis (Lattice)  ***
elseif method == 4
   px = input(' Enter row order of two-dimensional autoregressive process: ');
   py = input(' Enter column order of two-dimensional autoregressive process: ');
   disp(' ');
   disp(' ***** COMPUTING 2-D SPECTRUM BASED ON SELECTIONS *****');  
   [rho_Q1,a_Q1,rho_Q4,a_Q4,rho,A] = lattice_2D(px,py,x);
   psd_type = 3; %input(' Enter psd type (1--Q1, 2--Q4, 3--combined Q1 & Q4): ');
   psd2D = ar_psd_2D(psd_type,num_psdx,num_psdy,Tx,Ty,rho_Q1,a_Q1,rho_Q4,a_Q4);

%***  Two-Dimensional Minimum Variance Spectral Analysis  ***
elseif method == 5
   px = input(' Enter row order of 2-D adaptive minvar filter: ');
   py = input(' Enter column order of 2-D adaptive minvar filter: ');
   disp(' ');
   disp(' ***** COMPUTING 2-D SPECTRUM BASED ON SELECTIONS *****');  
   psd2D = minimum_variance_psd_2D(num_psdx,num_psdy,px,py,Tx,Ty,x);
   
else
    error(' No method associated with selected number.')
end
disp(' ');

%*****  3-D and 2-D Plots of Spectral Estimation Results *****
disp(' ***** PLOT 2-D SPECTRAL RESULTS *****')
plot_psd_2D
%