% Script:     spectrum_demo_MC
% Copyright (c) 2019 by S. Lawrence Marple Jr.

% This script demonstrates six multichannel spectral estimation algorithms using
% two synthetic and one actual 2-channel signal (sunspots and temperature)

disp(' >> CONSULT THE README FILE BEFORE FIRST USE OF THIS SCRIPT <<');
disp(' Press space bar to continue. ');  % 6 June 2019

%*********************************  Data Source  *******************************
disp(' ');
disp(' This script demonstrates MULTI-CHANNEL spectral analysis.');
disp(' ');
disp(' ***** MULTICHANNEL DATA SOURCES *****')
disp(' Data choices for 2-channel spectral analysis demonstration:')
disp('    1 -- Real-valued 2-channel AR(1) test case [Section 15.12 of course text]')
disp('    2 -- Complex-valued 2-channel 3-sinusoids [2 common, 1 not common] in white noise')
disp('    3 -- Filtered sunspot numbers and St Louis temperatures for years 1855 to 1968')
choice = input(' Enter number of data source choice: ');
if  choice == 1
    % Following code generates real two-channel AR(1) test case [Sec 15.12]
    A = [.85 -.75; .65 .55]';
    y = [0 0];
    x = [];
    for k=1:100
        y = y*A + randn(1,2);
        x = [x; y];
    end
    T = 1;
    Fs = 1/T;
    num_psd = 1024;
    num_chs = 2;
elseif choice == 2
    % Following generates synthetic 2-channel complex signal with 3-sinusoids
    % per channel with additive white noise. Two sinusoids are common to each
    % channel but delayed with respect to the other channel. Third sinusoid
    % in each channel is independent of third sinusoid in other channel.
    Fs = 1000;  % Sampling rate
    T = 1/Fs;   % Sampling interval
    x = randn(512,2)+1i*randn(512,2);  % Additive white noise
    t = T*(0:511)';
    x(:,1) = x(:,1) + 10*exp(-1i*2*pi*25*t) + 10*exp(1i*2*pi*20*(t-20*T)) + 10*exp(1i*2*pi*200*t);
    x(:,2) = x(:,2) + 10*exp(-1i*2*pi*25*(t+15*T)) + 10*exp(1i*2*pi*20*t) + 10*exp(1i*2*pi*300*t);
    num_psd = 1024;
    num_chs = 2;
elseif choice == 3
    % Prepare sunspot_numbers as first column vector and temperatures as
    % second column vector of two samples per year for 1855 to 1968
    
    % Sunspot number preparation
    load('sunspot_numbers.dat')
    temporary = sunspot_numbers(:,2:13);
    [m,n] = size(temporary);
    xx = reshape(temporary.',m*n,1); % shapes sunsplot into column vector
    clear temporary
    f = [0 .5/6 1/6 1]; % LFP filter design parameters for sunspot numbers
    a = [1 1 0 0];
    wt = [1 10];
    b = firpm(48,f,a,wt);  % Parks-McClellan 49 LPF coefficients
    %[h,w] = freqz(b,1,1024);
    %plot(6*w/pi,20*log10(abs(h)))
    %title('sunspot LPF response')
    cc = conv(xx,b,'valid'); % LPF the original sunspot numbers
    yy = cc(1:6:length(cc)); % decimate sunspot record: 12/year to 2/year
    f = [0 0 .0455 1]; % HPF filter design parameters
    a = [0 0 1 1];
    wt = [10 1];
    b = firpm(36,f,a,wt);  % Parks-McClellan 37 HPF coefficients
    %[h,w] = freqz(b,1,1024);
    %plot(w/pi,20*log10(abs(h)))
    %title('sunspot HPF response')
    d1 = conv(yy,b,'valid'); % HPF the LPF and decimated sunspot numbers
    
    % St Louis temperature preparation
    load('temperature.dat')
    [m,n] = size(temperature);
    xx = reshape(temperature.',m*n,1); % shapes temperature as column vector
    clear temperature
    f = [0 0 1/6 1]; % LPF filter design parameters for temperature data
    a = [1 1 0 0];
    wt = [1 100];
    b = firpm(24,f,a,wt); % Parks-McClellan 25 LPF coefficients
    %[h,w] = freqz(b,1,1024);
    %plot(6*w/pi,20*log10(abs(h)))
    %title('temperature LPF design')
    cc = conv(xx,b,'valid'); % LPF the original temperature data
    yy = cc(1:6:length(cc)); % decimate temperature record: 12/year to 2/year
    f = [0 0 .0455 1]; % HPF filter design parameters
    a = [0 0 1 1];
    wt = [1000 1];
    b = firpm(36,f,a,wt);  % Parks-McClellan 37 HPF coefficients
    %[h,w]=freqz(b,1,1024);
    %plot(w/pi,20*log10(abs(h)))
    %title('temperature HPF filter response')
    d2 = conv(yy,b,'valid'); % HPF the LPF and decimated temperature data
    
    % Form 2-channel data record and align year data Jan 1855 to Jul 1968
    x = [d1(289:516) d2];
    num_psd = 1024;
    num_chs = 2;
    Fs = 2; % two samples/year
    T = 1./Fs;
else
    error(' No data source for this number.')
end
disp(' ');

%**********************  Multichannel Spectral Analysis  ***********************

disp(' ***** MULTICHANNEL SPECTRAL TECHNIQUE *****')
disp(' Multichannel spectral analysis technique choices:')
disp('      1 -- multichannel Welch periodogram method')
disp('      2 -- multichannel Blackman-Tukey correlogram method')
disp('      3 -- multichannel autoregressive Yule-Walker method')
disp('      4 -- multichannel autoregressive Vieira-Morf (MC lattice) method')
disp('      5 -- multichannel autoregressive Nuttall-Strand (MC lattice) method')
disp('      6 -- multichannel minimum variance PSD method')
method = input(' Enter number of algorithm choice: ');
disp(' ')
if  (method < 1) || (method > 7)
    error(' No method associated with number.')
end
disp(' ***** SPECTRAL PARAMETERS *****')

             %***  Multichannel Periodogram Spectral Analysis  ****
if  method == 1
    disp(' Window choices:  0 -- none , 1 -- Hamming , 2 -- Nuttall')
    window = input(' Enter number of window choice: ');
    seg_size = input(' Enter segment size per periodogram (# samples): ');
    if seg_size > num_psd
        error([' Segment size cannot exceed NUM_PSD = ',int2str(num_psd)])
    end
    overlap = input(' Enter inter-segment overlap (# samples): ');
    if (overlap < 0) || (overlap >= seg_size)
        error([' Segment overlap outside of range 0 <= overlap < ',...
            int2str(seg_size)])
    end
    psd = periodogram_psd_MC(num_psd,window,overlap,seg_size,T,x);

            %***  Multichannel Blackman-Tukey Spectral Analysis  ***
elseif  method == 2
    disp(' Window choices:  0 -- none , 1 -- Hamming , 2 -- Nuttall')
    window = input(' Enter number of window choice: ');
    max_lag=input(' Enter maximum lag (# samples) for estimated correlation: ');
    k = fix((num_psd -1)/2);
    if  max_lag > k
        error([' Maximum ACS lag cannot exceed ',int2str(k)])
    end
    psd = correlogram_psd_MC(num_psd,window,max_lag,T,x);

            %***  Multichannel Autoregressive Spectral Analysis  ***
elseif  (method == 3) || (method == 4) || (method == 5)
    p = input(' Enter order of multichannel autoregressive process: ');
    if method == 3
        [rho_a,a,rho_b,b] = yule_walker_MC(p,x); % MC Yule-Walker
    end
    if method == 4
        [rho_a,a,rho_b,b] = lattice_MC('VM',p,x); % MC Vieira-Morf
    end
    if method == 5
        [rho_a,a,rho_b,b] = lattice_MC('NS',p,x); % MC Nuttall-Strand
    end
    f_or_b = input(' Select: 1--forward MC AR PSD , 2--backward MC AR PSD: ');
    psd = ar_psd_MC(f_or_b,num_psd,T,rho_a,a,rho_b,b);

           %***  Multichannel Minimum Variance Spectral Analysis  ***
else
    p = input(' Enter order of minimum variance multichannel filter: ');
    psd = minimum_variance_psd_MC(num_psd,p,T,x);
end
disp(' ');

%*****  Two-Channel Case: Save and Plot Autospectra, MSC, Phase of MSC  *****

disp(' ***** PLOT 2-CHANNEL RESULTS *****');
if num_chs == 2
    psd_ch1 = real(psd(1:2:num_psd*2-1,1));
    psd_ch2 = real(psd(2:2:num_psd*2,2));
    temp = psd(1:2:num_psd*2-1,2);
    msc = ((real(temp).^2)+(imag(temp).^2))./(psd_ch1.*psd_ch2);
    threshold1 = 0;
    threshold2 = 0;
    if method == 2
        threshold1 = max(abs(psd_ch1))*10^(-4.5);
        threshold2 = max(abs(psd_ch2))*10^(-4.5);
    end
    vec1 = find(psd_ch1 <= threshold1);
    vec2 = find(psd_ch2 <= threshold2);
    vec3 = find(msc > 1.0);
    msc(vec1) = 0;
    msc(vec2) = 0;
    msc(vec3) = 0;
    coh_phase = angle(temp);
    coh_phase(vec1) = 0;
    coh_phase(vec2) = 0;
end
plot_psd_MC
%